import json
from pathlib import Path
from typing import List, Optional

from aiofiles import open
from httpx import AsyncClient

from .classes import BooruNotFound, Post, Site


async def resolve_site(site: str) -> Optional[Site]:
    """Function to find a from sites.json from an aliases or site name

    :param site: Name of site or alias
    :return: Cunnypy Site Object
    """
    site = site.lower()
    async with open(f"{Path(__file__).parent}/sites.json", "r", encoding="utf-8") as f:
        raw = await f.read()
        sites = json.loads(raw)

        for domain, info in sites.items():
            if site == domain or site == info.get("domain") or site in info.get("aliases"):
                return Site(domain, info)
        raise BooruNotFound("That booru does not exist")


async def get_posts(site: Site, params: dict) -> Optional[List[Post]]:
    """Function to fetch posts from a given Site

    :param site: Cunnypy Site object
    :param params: params to use with the http request
    :return: List of posts
    """
    __http = AsyncClient(follow_redirects=True)

    async with __http as c:
        res = await c.get(site.search_url, params=params)
        res.raise_for_status()

        # Thanks e621
        if res.text == "":
            return []

        data = res.json()

        # Because sites like doing things different
        if site.domain == "e621.net":
            data = data["posts"]
        if site.domain == "gelbooru.com":
            data = data["post"]

        # No posts found
        if not data:
            return []

        posts = [Post(p, site) for p in data]
        return posts



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['cunnypy']

package_data = \
{'': ['*']}

install_requires = \
['aiofiles>=22.1.0,<23.0.0', 'httpx>=0.23.0,<0.24.0']

setup_kwargs = {
    'name': 'cunnypy',
    'version': '1.0.0',
    'description': 'A python library for several image boards',
    'long_description': '<h1 align="center">🦀 Cunny.py 🦀</h1>\n\n<h3 align="center">A python library for several image boards</h3>\n\n<p align="center">\n    <a href="https://liberapay.com/GlitchyChan/donate">\n        <img src="https://img.shields.io/badge/Liberapay-F6C915?style=for-the-badge&logo=liberapay&logoColor=black" alt="liberapay" />\n    </a>\n    <a href="https://discord.gg/ZxbYHEh">\n        <img src="https://img.shields.io/badge/Discord-5865F2?logo=discord&logoColor=fff&style=for-the-badge" alt="Discord" />\n    </a>\n    <a href="https://twitter.com/glitchychan">\n        <img src="https://img.shields.io/badge/twitter-%2300acee?&style=for-the-badge&logo=twitter&logoColor=white" alt="twitter" />\n    </a>\n</p>\n\n---\n\n<p align="center">\n    <a href="#about">About</a> •\n    <a href="#features">Features</a> •\n    <a href="#usage">Development</a>\n</p>\n\n## **About**\nThis library is to make it much easier to interact with image boards with python.\n\n## **Features**\n- 🔁 Fully Async\n- 🔥 Blazingly Fast™️\n- 💯 Supports many sites with aliases (see [sites.json](./cunnypy/sites.json))\n- 🎱 Supports random search\n- ⚙️ 1 simple import to get going\n\n## **Usage**\nTo get started with cunny.py simply import the library and use the search function\n\nExample search with gelbooru\n```python\nimport cunnypy\n\nposts = await cunnypy.search("gel", tags=["megumin"])\nprint(posts)\n```\nThis will print out a list of [Post](./cunnypy/classes.py#L30-L80) classes which you can then manipulate\n\n\nOther examples can be found in the [Examples](./examples) folder\n',
    'author': 'Glitchy',
    'author_email': 'thepatheticweebgamer@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://codeberg.org/CunnyTech/Cunnypy',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



import random
from typing import Dict, List, Optional

from .classes import CunnyPyError, Post, Site
from .utils import get_posts, resolve_site


async def search(
    site: str,
    credentials: Optional[Dict] = None,
    tags: Optional[List[str]] = None,
    limit: Optional[int] = 10,
    page: Optional[int] = 1,
    gatcha: Optional[bool] = False,
) -> Optional[List[Post]]:
    """Search the booru you have chosen

    :param site: The booru you wish to search
    :param credentials: Any api credentials for the booru
    :param tags: Any tags you are wanting to search with
    :param limit: Limit of posts to fetch
    :param page: Which page to fetch posts from
    :param gatcha: Weather to get a random post or not
    :return: List of CunnyPy Posts or just one Post
    """

    # Get site
    site: "Site" = await resolve_site(site) if not isinstance(site, Site) else site

    # Hard limit
    if limit >= 1000:
        raise CunnyPyError("Limit cannot exceed 1000")

    # Gatcha related params
    if gatcha and site.random:
        tags.append("order:random")

    # Format tags
    tags = [tag.strip().lower().replace(" ", "_") for tag in tags]

    # Setup httpx params
    params = {"tags": tags, "limit": limit, "pid": page}
    if credentials:
        params.update(credentials)

    # Fetch the posts
    posts = await get_posts(site, params)

    return [random.choice(posts)] if gatcha else posts



import reprlib
from datetime import datetime
from typing import List

from aiofiles import open
from httpx import AsyncClient


class CunnyPyError(BaseException):
    pass


class BooruNotFound(CunnyPyError):
    pass


class Site:
    """Class to represent one of the sites"""

    def __init__(self, domain: str, info: dict):
        self.domain: str = domain
        self.aliases: List[str] = info.get("aliases")
        self.nsfw: bool = info.get("nsfw")
        self.search_url: str = f"https://{self.domain}{info.get('api').get('search')}"
        self.post_view: str = f"https://{self.domain}{info.get('api').get('postView')}"
        self.post_comments_view: str = f"https://{self.domain}{info.get('api').get('postComments')}"
        self.random: bool = info.get("random")


class Post:
    """Class to represent a post from an image board"""

    def __init__(self, payload: dict, site: Site):
        # Cross compatability with older Booru API's
        payload = {k.strip("@"): v for k, v in payload.items()}

        self.id = payload.get("id", 0)
        self.creator_id = payload.get("creator_id", 0)
        self.created_at = payload.get("created_at")
        self.file_url = payload.get("file_url")
        self.filename = payload.get("image")
        self.source = payload.get("source") or None
        self.hash = payload.get("md5")
        self.height = payload.get("height")
        self.width = payload.get("width")
        self.rating = payload.get("rating")
        self.has_sample = payload.get("has_sample", "false")
        self.has_comments = payload.get("has_comments", "false")
        self.has_notes = payload.get("has_notes", "false")
        self.has_children = payload.get("has_children", "false")
        self.tags = payload.get("tags")
        self.change = datetime.fromtimestamp(int(payload.get("change", 0)))
        self.directory = payload.get("directory")
        self.status = payload.get("status")
        self.locked = payload.get("post_locked", 0)
        self.score = payload.get("score", 0)
        self.post_view = f"{site.domain}{site.post_view}{self.id}"
        self._payload = payload
        self._site = site

    async def download(self, location: str) -> None:
        """Allows you to download the image of the post to a specified location

        :param location: Location to download to
        """
        async with open(f"{location}/{self.filename}") as f:
            async with AsyncClient() as client:
                res = await client.get(self.file_url)
                await f.write(str(res.content))

    def __str__(self):
        return self.file_url

    def __int__(self):
        return self.id

    def __repr__(self):
        rep = reprlib.Repr()
        return f"<Post(id={self.id}, filename={rep.repr(self.filename)}, owner={rep.repr(self.creator_id)})>"
